include <rounded-corner-plate.scad>
include <vslot-insert.scad>
include <hexnut.scad>

// Models for printing wire covers and such

module bent_shape(dia,len) {
   union() {
      sphere(r=dia/2,$fn=64);
      rotate([90,0,0])
         cylinder(r=dia/2,h=len,$fn=64);
      rotate([180,0,0])
         cylinder(r=dia/2,h=len,$fn=64);
      translate([-dia/2,-len,-len])
      cube([dia,len,len]);
   }
}

// A "covered walkway" for wires to run through before going through
// a hole in the top of the box (keep stuff from falling directly
// down hole and protect wires).
//
module right_angle_cover(
   dia=25.4/2,                  // diameter of half round opening
   wall=2,                      // thickness of cover wall
   base=3,                      // thickness of cover base
   len=20,                      // length of straight part before hole
   tab_hole=3.4,                // diameter of bolt hole in mounting tab
   tab_margin=2,                // extra margin around tab hole
   left_tab=true,               // print mounting tab on left
   right_tab=true,              // print mounting tab on right
   rear_tab=true) {             // print mounting tab at rear

   tab_width=tab_hole+2*tab_margin;
   tot_width=tab_width+wall+(dia/2);

   translate([0,0,base])
   difference() {
      union() {
         bent_shape(dia+2*wall,len);
         if (left_tab) {
            translate([-tot_width/2,-len/2,-base])
               rounded_corner_plate(tot_width,len,base,base);
         }
         if (right_tab) {
            translate([tot_width/2,-len/2,-base])
               rounded_corner_plate(tot_width,len,base,base);
         }
         if (rear_tab) {
            rlen=len+((dia+2*wall)/2)+tab_width;
            translate([0,(rlen/2)-len,-base])
               rounded_corner_plate(dia+2*wall,rlen,base,base);
         }
      }
      bent_shape(dia,len+2);
      translate([-(dia+2*wall+2)/2,-(len+1),-(len+2+base)])
         cube([dia+2*wall+2,len+dia+2,len+2]);
      if (left_tab) {
         translate([-(dia/2+tab_hole/2+wall+tab_margin),-len/2,-(base+1)])
            cylinder(r=tab_hole/2,h=base+2,$fn=64);
      }
      if (right_tab) {
         translate([(dia/2+tab_hole/2+wall+tab_margin),-len/2,-(base+1)])
            cylinder(r=tab_hole/2,h=base+2,$fn=64);
      }
      if (rear_tab) {
         translate([0,(dia+2*wall)/2+tab_hole/2+tab_margin,-(base+1)])
            cylinder(r=tab_hole/2,h=base+2,$fn=64);
      }
   }
}

// A 16.8mm high model of the conduit tubing I've got. Created by scanning
// a section of conduit which was cut in half longways and using gimp
// to manipulate that into a svg file that can be imported and rotated
// to generate this model.
//
module conduit() {
   rotate_extrude(angle=360,$fn=64)
      import("conduit.svg");
}

// A conduit connector to go on bottom of the top of the box and poke up
// through a 7/8" hole (the smallest forstner bit I have larger than the
// conduit). The conduit model scaled up by 15% seems to produce a nice
// tight fit the conduit won't pop out of once it is fully unfolded
// inside the hole.
//
module conduit_base() {
   // slightly smaller than 7/8"
   cyl_dia=((7/8)*25.4)-0.4;
   // big enough for M3 mounting holes leaving room for bolt heads
   base_dia=cyl_dia+((5.5+2+2)*2);
   difference() {
      union() {
         cylinder(r=cyl_dia/2,h=16,$fn=64);
         cylinder(r=base_dia/2,h=2.5,$fn=64);
      }
      translate([0,0,-0.4])
         scale([1.15,1.15,1])
            conduit();
      translate([0,(base_dia/2)-(2+(5.5/2)),-1])
         cylinder(r=(3.4)/2,h=2.5+2,$fn=64);
      translate([0,-((base_dia/2)-(2+(5.5/2))),-1])
         cylinder(r=(3.4)/2,h=2.5+2,$fn=64);
      translate([cyl_dia/2,-base_dia/2,-1])
         cube([base_dia,base_dia,2.5+2]);
      translate([-(base_dia+cyl_dia/2),-base_dia/2,-1])
         cube([base_dia,base_dia,2.5+2]);
   }
}

// A vslot insert with an M2 nut trap and a slot for turning it into
// position with a screwdriver.
//
module conduit_top_insert() {
   sdd=0.8;
   rotate([90,0,0])
   difference() {
      vslot_insert_base();
      // subtract slot in top for screwdriver
      translate([-1.1/2,-5,5.87-sdd])
         cube([1.1,10,3]);
      translate([0,0,-1])
         cylinder(r=(2.4/2),h=6+2,$fn=64);
      translate([0,0,1.5+(m2nut_thick+0.3)])
      union() {
         rotate([0,0,30])
            hexnut(m2nut_flat+0.3,m2nut_thick+0.3);
         translate([-(m2nut_flat+0.3)/2,0,-(m2nut_thick+0.3)])
            cube([m2nut_flat+0.3,5,m2nut_thick+0.3]);
      }
   }
}


// Utility to make the t-nut/vslot grabber for the conduit_top to use.
//
module tnutslot(ylen=20,holepos=10) {
   difference() {
      union() {
         translate([0,0,2])
            cube([4.5,ylen,16]);
         translate([0,0,0])
            cube([3,ylen,20]);
      }

      translate([10+3,ylen+1,10])
         rotate([90,0,0])
            vslot20x20(ylen+2);

      translate([-10,holepos,10])
         rotate([0,90,0])
            cylinder(r=(5.2/2), h=20, $fn=64);
   }
}

// For the top of the conduit, attach a conduit cylinder like the base
// using a t-nut and v-slot arrangement.
//
module conduit_top(do_right=false) {
   cyl_dia=((7/8)*25.4);
   m2_head=3.7;
   ext=15;
   mirror([do_right?1:0,0,0])
   difference() {
      union() {
         cylinder(r=cyl_dia/2,h=16,$fn=64);
         translate([3-cyl_dia/2,0,0])
            mirror([1,0,0])
               difference() {
                  tnutslot(20,15);
                  translate([-5,-5,16])
                     cube([40,40,40]);
               }
      }
      translate([0,0,-0.4])
         scale([1.15,1.15,1])
            conduit();
   }
}

// Having measured the position of the conduit_top when it is positioned to
// avoid all the nasty x,y,z moving parts, I need the conduit bottom to be
// as close to directly below it as I can get. This is a little piece to
// print showing the center and outer ring of the hole I want when
// positioned in the corner of the frame. (As near as I can tell, this
// worked pretty well, time will show if I got it right).
//
module conduit_drill_guide() {
   xoff=9.5+18;
   yoff=11.125;
   cyl_dia=((7/8)*25.4);
   difference() {
      union() {
         difference() {
            cube([2*xoff,3*yoff,3]);
            translate([xoff,yoff+0.4,-1])
               cylinder(r=cyl_dia/2,h=5,$fn=64);
         }
         translate([xoff-1.5,yoff,0])
            cube([3,yoff+1,3]);
         translate([xoff,yoff+0.4,0])
            cylinder(r=5/2,h=3,$fn=64);
      }
      translate([xoff,yoff+0.4,-1])
         cylinder(r=3/2,h=5,$fn=64);
   }
}
