// A direct drive bowden extruder with a v-slot rail mount. (Can also wire
// tie the motor to the rail for extra support and to make good contact for
// any extra heat dissipation that might provide).

include <rounded-corner-plate.scad>
include <rounded-slot.scad>
include <hexnut.scad>

// I like the bowden threaded adapter from the Berry Tripper, so I'm
// stealing three existing .stl files from it:
//
// bowden_cap.stl
// bowden_ring.stl
// bowden_threaded_adapter.stl
//
// The dimensions of the thread adapter are:
//
//             X      Y     Z
// Minimum: -11.00 -41.80  0.00
// Maximum:  12.00 -27.77 26.63
// Size:     23.00  14.03 26.63
//
// The center of the hole is 11mm from one end and 12mm from the other

// The hobbed goblin gear can go on the motor anywhere from 7-16mm from
// face of motor, or 10-20mm (depending on which way I face the gear).
// Note that the set screw sticks out quite a bit, so I'll need to leave
// room for it to rotate as well.

// This seems like sticking the long end of the adapter against the motor
// face may be the way to go and arrange for the hobbed section of gear
// to be at 12mm off the face of the motor.

// Diameter of hobbed gear looks like 8.5mm, and if I place a piece of
// 1.75mm filament in the hobbed groove and measure the diameter plus
// the fllament, I get 9.68. Without the hobb, that would be 10.25,
// so the filament seems to go down 0.57mm into the groove (without being
// squeezed any).
//
// So I want the hole for the filament guide to be this far off center:
//
// (8.5/2)-0.57
//
// Meanwhile, the diameter plus the set screw height is:
//
// 9.86, so the set screw sticks up an extra 1.36mm, so I'll need to add at
// least that much to the radius of the hole for the shaft to rotate in.

// Diameter of hobbed gear body
hobb_dia=8.5;

// Offset where bowden tube should be located to run the filament
// across the hobbed area of the gear.
bowden_tube_offset=(hobb_dia/2)-0.57;

// How far above the face of the motor the center of the hobbed area
// of the gear should be located.
hobb_height=12;

// Dimensions for nema 17 motor
nema_screw_sep=31;
nema_disk_height=2;
nema_disk_dia=22;
nema_motor_width=42.3;

// Dimensions for skate bearing
skate_od=22;
skate_id=8;
skate_thick=7;

// Offset from center of motor to pivot point for the lever arm that
// holds the skate bearing.

pivot_x=24;
pivot_y=30;

pivot_extra=pivot_x-(nema_motor_width/2);

// Dimensions for the lever holding the skate bearing

lever_width=12;
lever_length=nema_motor_width+2*pivot_extra+lever_width/2;

// Offset from center of motor to center of skate bearing
bearing_y=skate_od/2+bowden_tube_offset+(1.75/2);

// Thickness of the supports for the bearing
support_thick=4;

// Total thickness of the lever arm supports
lever_thick=(2*support_thick)+(skate_thick+0.2);

// X offset of the tension screw
screw_head_x=-(2+nema_motor_width/2);

// Spacing for t-nut holes to allow each hole to contain a t-nut
t_nut_spacing=15.5;

// Extra redius to provide so the set screw in the hobbed gear has space
// to rotate
//
// NOTE: 1.4 wasn't big enough - I got a little scraping. 
set_screw_extra=1.6;

// My 30mm screws can't go all the way into the holes on the motor,
// so I need some short spacers to tighten the screws against.
spacer=1.3;

module screw_spacer() {
   difference() {
      cylinder(r=7/2,h=spacer,$fn=64);
      translate([0,0,-1])
         cylinder(r=3.2/2,h=spacer+2,$fn=64);
   }
}

// M3 washer dimensions
M3_wash_thick=0.47;
M3_wash_dia=6.77;

// M5 washer dimensions
M5_wash_thick=0.94;
M5_wash_dia=9.84;

// Build a disk with a nut trap to use to adjust spring tension
//
// This newer version allows the nut, and two washers to be
// glued into the knob, making it much simpler to take the knob on and off
// as I had to do that frequently when first setting up the printer.
//
module tension_nut() {
   difference() {
      // main body of tension nut
      cylinder(r=22/2,h=1.5+m3nut_thick+M3_wash_thick+M5_wash_thick+1.5,$fn=64);
      // Combined shape of M3 nut, M3 washer, and M5 washer stacked up with a
      // 3mm hole punched through all of them.
      translate([0,0,1.5])
         union() {
            translate([0,0,m3nut_thick+0.1])
               hexnut(m3nut_flat+0.2,m3nut_thick+0.1);
            translate([0,0,m3nut_thick])
               cylinder(r=(M3_wash_dia+0.3)/2,h=M3_wash_thick+0.1,$fn=64);
            translate([0,0,m3nut_thick+M3_wash_thick])
               cylinder(r=(M5_wash_dia+0.3)/2,h=10*M5_wash_thick,$fn=64);
            translate([0,0,-25])
               cylinder(r=3.2/2,h=50,$fn=64);
         }
         // notch edges for poor mans knurling
         for ( i = [0 : 45 : 315] ) {
            rotate([0,0,i])
               translate([22/2-1.5,-3/2,-1])
                  cube([5,3,20]);
         }
   }
}

// The shape for the supports which will hold the skate bearing
//
module bearing_support(extra=0) {
   translate([0,bearing_y,hobb_height-lever_thick/2])
      hull()
         union() {
            cylinder(r=(skate_id+1+extra)/2, h=lever_thick, $fn=64);
            translate([-(skate_od+0.5+extra)/2,
                       (pivot_y-bearing_y)-lever_width/2,0])
               cube([skate_od+0.5+extra,lever_width,lever_thick]);
         }
}

// The straight lever arm which holds the supports which hold the bearing
//
// NOTE: The arm was thicker than it ought to be, needing some filing
// to get it to fit in the slots. Could make it thinner.
//
module lever_arm() {
   difference() {
      translate([pivot_x,pivot_y,hobb_height-lever_thick/2])
         difference() {
            union() {
               cylinder(r=lever_width/2,h=lever_thick,$fn=64);
               translate([-lever_length,-lever_width/2,0])
                  cube([lever_length,lever_width,lever_thick]);
            }
            translate([0,0,-1])
               cylinder(r=5.2/2,h=lever_thick+2,$fn=64);
         }

      // Make hole that matches the position of the tension screw mount
      // on the extruder body. Screw will go through here then have spring
      // compressed on the far side.
      translate([screw_head_x,pivot_y-(lever_thick/2),hobb_height])
         rotate([-90,0,0])
            rotate([0,0,90])
               rounded_slot(2,3.2,lever_thick+2);

      // Make M3 hole through the lever arm to use to secure the two
      // halfs together after they are printed separately
      translate([-17,pivot_y,hobb_height-(lever_thick+2)/2])
         cylinder(r=3.2/2,h=lever_thick+2,$fn=64);
   }
}

// Bearing shaped hole to subtract from the support to make the
// lever arm piece.
//
// NOTE: The "washers" needed a lot of filing down to get the arm
// to fit - I should make them thinner.
//
module sub_wheel() {
   th=0.6;         // thickness of printed "washers"
   translate([0,bearing_y,hobb_height-(skate_thick+2*th)/2])
   difference() {
      // The outer wheel shape with extra space to leave room to
      // rotate without scraping anywhere
      cylinder(r=(skate_od+0.5)/2,h=skate_thick+2*th,$fn=64);
      // The 8mm shaft (a little small) to be printed to hold the
      // center of the bearing.
      translate([0,0,-1])
         cylinder(r=(skate_id-0.1)/2,h=2+skate_thick+2*0.6,$fn=64);
      // The top "washer"
      translate([0,0,skate_thick+2*th-th])
         cylinder(r=(skate_id+2)/2,h=1+th,$fn=64);
      // The bottom "washer"
      translate([0,0,-1])
         cylinder(r=(skate_id+2)/2,h=1+th,$fn=64);
   }
}

// The printable bottom half of the bearing arm
//
module bottom_bearing_arm() {
   difference() {
      translate([0,-bearing_y,-(hobb_height-lever_thick/2)])
         difference() {
            union() {
               lever_arm();
               bearing_support();
            }
            sub_wheel();
         }
      translate([-50,-50,lever_thick/2])
         cube([100,100,50]);
   }
}

// The printable top half of the bearing arm
//
module top_bearing_arm() {
   mirror([0,1,0])
      bottom_bearing_arm();
}

// A circular arc to be subtracted out of the body to give room for
// the skate bearing to rotate into place when swinging the lever arm.
// Computed mainly by looking at preview and adjusting numbers till
// it looked good - no science involved.
//
module large_arc() {
   rotate([0,0,31])
      difference() {
         rotate([0,0,-31])
            difference() {
               cylinder(r=39.14,h=skate_thick+0.2,$fn=64);
               translate([0,0,-1])
                  cylinder(r=30,h=2+skate_thick+0.2,$fn=64);
               translate([-40,0,-1])
                  cube([80,80,2+skate_thick+0.2]);
            }
         translate([-40,-80,-1])
            cube([80,80,2+skate_thick+0.2]);
      }
}

// Like large_arc but for the smaller support structure clearence.
//
module small_arc() {
   rotate([0,0,33])
      difference() {
         rotate([0,0,-33])
            difference() {
               cylinder(r=35,h=lever_thick,$fn=64);
               translate([0,0,-1])
                  cylinder(r=25,h=2+lever_thick,$fn=64);
               translate([-40,0,-1])
                  cube([80,80,2+lever_thick]);
            }
         translate([-40,-80,-1])
            cube([80,80,2+lever_thick]);
      }
}

// Subtract this from the body to leave a hole where the bearing
// and supports can swing into place
//
// NOTE: Not sure this is quite big enough. The arm won't rotate into
// the body as far as I thought it might.
//
module bearing_hole() {
   translate([0,-0.5,0])
      union() {
         hull()
            union() {
               bearing_support(extra=0.5);
               translate([pivot_x,pivot_y,hobb_height-lever_thick/2])
                  small_arc();
            }
         translate([0,bearing_y,hobb_height-(skate_thick+0.5)/2])
            union() {
               hull()
                  union() {
                     cylinder(r=(skate_od+0.5)/2,h=skate_thick+0.5,$fn=64);
                     translate([pivot_x,pivot_y-bearing_y,0])
                        large_arc();
                  }
               translate([-(skate_od+0.5)/2,0,0])
                  cube([skate_od+0.5,(skate_od+0.5)/2,skate_thick+0.5]);
               translate([-6/2, -(2+(skate_od+0.5)/2), 0])
                  cube([6,5,skate_thick+0.5]);
            }
      }
}

// Just a module to visualize the completed lever arm and bearing assembly
// when I needed to look at things to see how they fit.
//
module view_bearing() {
   union() {
      bearing_support();
      lever_arm();
      translate([0,bearing_y,hobb_height-(skate_thick+0.2)/2])
         cylinder(r=(skate_od+0.2)/2, h=skate_thick+0.2, $fn=64);
   }
}

// Make a shape for a hole to hold the head of the M3 tension screw
//
// NOTE: The 8mm cylinder isn't big enough to hold a screw with
// a washer. Should probably make it both wider and deeper (which might
// mean cutting through the back, but that shouldn't be a problem
// in practice.
//
module screw_head() {
   translate([screw_head_x,(nema_motor_width/2)-(3.6+2),hobb_height])
   rotate([-90,0,0])
      union() {
         cylinder(r=8/2,h=3.6,$fn=64);
         rotate([0,0,90])
            rounded_slot(2,3.2,3.6+2+0.1);
      }
}

// Shape of the adapter to make a hole for it to slide into...
//
module adapter_hole(want_width=14.0, want_height=11.1, want_len=23) {
   translate([0,want_width/2,0])
      rotate([0,0,-90])
         linear_extrude(height=want_len)
            scale([want_width/367, want_height/293, 1])
               polygon([[0,0],
                        [367,0],
                        [367,87],
                        [283,172],
                        [283,206],
                        [342,206],
                        [342,293],
                        [26,293],
                        [26,206],
                        [85,206],
                        [85,171],
                        [0,87]]);
}

// Extension from the extruder body to hold the lever arm
//
module lever_bracket() {
   difference() {
      translate([pivot_x-3, pivot_y-6,0])
         rounded_corner_plate(18,25,hobb_height,3);
      translate([pivot_x,pivot_y,hobb_height-lever_thick/2])
         union() {
            cylinder(r=(lever_width+3)/2,h=lever_thick,$fn=64);
            translate([-30,-(lever_width+3)/2,0])
               cube([30,lever_width+3,lever_thick]);
            translate([-(lever_width+3)/2,0,0])
               cube([lever_width+3,30,lever_thick]);
         }
   }
}

// This is the common extruder body. A mirror of this forms the top
// body that flips over and screws onto the bottom body and the
// motor.
//
module extruder_body_common() {
   ll=(lever_length+lever_width/2)/2;
   lw=10;

   difference() {
      union() {
         // Start with plate that covers motor face
         rounded_corner_plate(nema_motor_width, nema_motor_width, hobb_height, 3);
         // Add in smaller extension to provide pivot point for lever arm
         lever_bracket();
         // Add in entension on top where the M3 tension screw head will sit
         translate([-ll, nema_motor_width/2-lw,0])
            cube([ll,lw,hobb_height]);
      }

      // Subtract off slots for the bowden_threaded_adapter to slide in
      // (offset them a bit extra so they are far enough out to not run into
      // the disk on the top of the nema 17 motor, but slide flush against
      // the motor face
      translate([-(2.5+((nema_motor_width/2)+0.1)),bowden_tube_offset,-1])
         adapter_hole(14.2,11.3,25);
      mirror([1,0,0])
         translate([-(2.5+((nema_motor_width/2)+0.1)),bowden_tube_offset,-1])
            adapter_hole(14.2,11.3,25);

      // Subtract off the 4mm hole for the bowden tube to slide through
      translate([-(nema_motor_width+2)/2,bowden_tube_offset,hobb_height])
         rotate([0,90,0])
            cylinder(r=4.2/2,h=nema_motor_width+2,$fn=64);

      // Subtract off the M3 holes to screw extruder to the motor
      translate([nema_screw_sep/2, nema_screw_sep/2, -1])
         cylinder(r=3.2/2,h=hobb_height+2,$fn=64);
      translate([-nema_screw_sep/2, nema_screw_sep/2, -1])
         cylinder(r=3.2/2,h=hobb_height+2,$fn=64);
      translate([nema_screw_sep/2, -nema_screw_sep/2, -1])
         cylinder(r=3.2/2,h=hobb_height+2,$fn=64);
      translate([-nema_screw_sep/2, -nema_screw_sep/2, -1])
         cylinder(r=3.2/2,h=hobb_height+2,$fn=64);

      // Subtract off hole for hobbed gear to fit in
      translate([0,0,-1])
         cylinder(r=(hobb_dia+0.2)/2,h=hobb_height+2,$fn=64);

      // Tricky stuff follows. Need to subtract off the space for the
      // skate bearing to fit, but it also needs to be able to rotate
      // into position, so I'm looking at a preview of the one_roller_hole()
      // sitting in place and coming up with some arcs and such to subtract
      // off the extruder body which look like they will provide enough
      // space.

      // The bearing_hole contains a lot of elaborate arcs and such
      // to (hopefully) leave space for the skate bearing to rotate
      // into position to press on the filament.

      bearing_hole();

      // Hole for M5 bolt for lever arm to pivot on
      translate([pivot_x, pivot_y, -1])
         cylinder(r=5.2/2, h=hobb_height+2, $fn=64);
      // Make sure there is space for the cap to rotate on the threaded
      // adapter
      translate([nema_motor_width/2,bowden_tube_offset,hobb_height])
         rotate([0,90,0])
            cylinder(r=22.5/2,h=50,$fn=64);
      translate([-nema_motor_width/2,bowden_tube_offset,hobb_height])
         rotate([0,-90,0])
            cylinder(r=22.5/2,h=50,$fn=64);
      // Subtract off the space for the tension screw head
      screw_head();
   }
}

// This is the extruder body that screws directly to the face of the motor
// It has the common parts plus the nema motor disk cutout and the top
// v-slot rail mount
//
module extruder_body_one() {
   difference() {
      union() {
         extruder_body_common();
         translate([-(6+10+0.1),-(20+nema_motor_width/2),0])
            union() {
               cube([6,21,hobb_height]);
               cube([6,20,2*t_nut_spacing]);
            }
      }

      // Subtract off the hole to leave room for the disk on top of motor
      translate([0,0,-1])
         cylinder(r=(nema_disk_dia+0.2)/2, h=1+nema_disk_height+0.2, $fn=64);

      // Subtract off the holes for the t-nut connections
      translate([0,-(10 + nema_motor_width/2), t_nut_spacing/2])
         rotate([0,-90,0])
            cylinder(r=5.2/2,h=nema_motor_width,$fn=64);
      translate([0,-(10 + nema_motor_width/2), t_nut_spacing+t_nut_spacing/2])
         rotate([0,-90,0])
            cylinder(r=5.2/2,h=nema_motor_width,$fn=64);
   }
}

// This is the extruder body that flips over and screws on top of the
// extruder_body_one. It has a larger central hole to provide room for the
// set screw on the hobbed gear to rotate and it has the bottom v-slot
// mount
//
module extruder_body_two() {
   difference() {
      union() {
         mirror([0,1,0])
            extruder_body_common();
         translate([10+0.1,nema_motor_width/2,0])
            union() {
               translate([0,-1,0])
                  cube([6,21,hobb_height]);
               cube([6,20,2*t_nut_spacing]);
            }
      }

      // Make the central hole bigger on top to leave room for
      // the set screw in the hobbed gear

      translate([0,0,-1])
         cylinder(r=set_screw_extra + hobb_dia/2,h=hobb_height+2, $fn=64);

      // Subtract off the holes for the t-nut connections
      translate([0,(10 + nema_motor_width/2), t_nut_spacing/2])
         rotate([0,90,0])
            cylinder(r=5.2/2,h=nema_motor_width,$fn=64);
      translate([0,(10 + nema_motor_width/2), t_nut_spacing+t_nut_spacing/2])
         rotate([0,90,0])
            cylinder(r=5.2/2,h=nema_motor_width,$fn=64);
   }
}
