// Smoothieboard mount for the interior of the box I'm building.
// Two independent supports for either end of the board with the
// holes positioned to match in each one and M4 size holes sticking
// out either side for bolting to the bottom of the box.
//
// Best way to line up the box holes is probably to screw the
// smoothieboard on, then find out where to drill the holes in the
// box using the whole assembly as a guide.

include <carriages.scad>
include <hexnut.scad>

// Subtract off these holes from the support to use M3 bolts on the
// smoothieboard itself with a nice nut trap built in so the nuts
// are easier to place exactly on the underside.
//
module smoothie_mount_hole() {
   union() {
      cylinder(r=1.8, h=plate_thick+2, $fn=64);
      translate([0,0,0.99+m3nut_thick])
         hexnut(m3nut_flat+0.3,m3nut_thick);
   }
}

// Top hole mount positions
//
module smoothie_top_holes() {
   union() {
      translate([3.6,102.5,-1])
         smoothie_mount_hole();
      translate([101.6,103,-1])
         smoothie_mount_hole();
   }
}

// Bottom hole mount positions
//
module smoothie_bottom_holes() {
   union() {
      translate([4.2,3.5,-1])
         smoothie_mount_hole();
      translate([96.7,10,-1])
         smoothie_mount_hole();
   }
}

// The support itself with either top or bottom holes punched
// through the support and cylinders provided for physical support
// across the board.
//
module smoothie_supports(do_top=true) {
   sl=134;
   ss=13;
   nc=4;
   fc=ss/2;
   lc=sl-fc;
   cs=(lc-fc)/(nc-1);
   ytran=do_top ? (-(ss/2)+102.75) : (-(ss/2)+6.75);
   rotate([0,180,0])
   difference() {
      translate([-(sl/2)+54.75,ytran,0])
         difference() {
            union() {
               cube([sl,ss,plate_thick]);
               for (i = [fc : cs : lc]) {
                  translate([i,ss/2,-plate_thick])
                     cylinder(r=(ss/2),h=plate_thick*2,$fn=64);
               }
            }
            translate([fc,ss/2,-(plate_thick+1)])
               cylinder(r=4.3/2,h=plate_thick*3,$fn=64);
            translate([lc,ss/2,-(plate_thick+1)])
               cylinder(r=4.3/2,h=plate_thick*3,$fn=64);
         }
      if (do_top) {
         smoothie_top_holes();
      } else {
         smoothie_bottom_holes();
      }
   }
}

// The one mounting hole next to the serial port pins is too close to the
// soldered in connector for the head of an M3 bolt to fit. Take the stl
// model of the smoothieboard I downloaded from somewhere (don't remember
// where it came from) and use it to subtract off the shape of the tiny
// crevice from a 4mm high cube to get a special "washer" I can run an M3
// bolt through (use 4mm so a 12 mm bolt will work on this corner in place
// of the 8mm bolts on the other corners). [This doesn't fit very well, but
// I only need one and some sanding and filing made it work.]
//
module smoothie_washer() {
   rotate([-90,0,0])
   difference() {
      cube([10,4,6]);
      translate([-118.8,-1.5,-11.85])
         import("/home/tom/3d/jiggit/smoothie/Smoothieboard_X5.stl");
      translate([7.1,5,2.65])
         rotate([90,0,0])
            cylinder(r=1.8,h=10,$fn=64);
   }
}
